@@ -5,10 +5,12 @@ git privacy
 import click
 import git  # type: ignore
 import os
+import shutil
 import stat
 import sys
 
 from datetime import datetime, timezone
+from pkg_resources import resource_stream, resource_string
 from typing import Optional, TextIO, Tuple
 
 
@@ -16,6 +18,7 @@ from . import GIT_SUBDIR
 from . import crypto as crypt
 from .cli import email
 from .cli import keys
+from .cli import pushcheck
 from .cli.utils import assertCommits
 from .dateredacter import DateRedacter, ResolutionDateRedacter
 from .encoder import (
@@ -26,7 +29,7 @@ from .rewriter import AmendRewriter, FilterRepoRewriter
 from .utils import fmtdate
 
 
-class GitPrivacyConfig(object):
+class GitPrivacyConfig:
     SECTION = "privacy"
 
     def __init__(self, gitdir: str) -> None:
@@ -114,11 +117,10 @@ def cli(ctx: click.Context, gitdir):
 @click.option('-g', '--global', "globally", is_flag=True,
               help="Setup a global template instead.")
 @click.option('--timezone-change', type=click.Choice(("warn", "abort")),
-              #default="warn", show_default=True,
               help=("Reaction strategy to detected time zone changes pre commit."
                     " (default: warn)"))
 @click.pass_context
-def do_init(ctx: click.Context, globally: bool, 
+def do_init(ctx: click.Context, globally: bool,
             timezone_change: Optional[str]) -> None:
     """Init git-privacy for this repository."""
     repo = ctx.obj.repo
@@ -129,6 +131,7 @@ def do_init(ctx: click.Context, globally: bool,
     copy_hook(git_dir, "post-commit")
     copy_hook(git_dir, "pre-commit")
     copy_hook(git_dir, "post-rewrite")
+    copy_hook(git_dir, "pre-push")
     # only (over-)write settings if option is explicitly specified
     if timezone_change is not None:
         assert timezone_change in ("warn", "abort")
@@ -160,8 +163,6 @@ def get_template_dir(repo: git.Repo) -> str:
 
 
 def copy_hook(git_path: str, hook: str, ) -> None:
-    from pkg_resources import resource_stream, resource_string
-    import shutil
     hookdir = os.path.join(git_path, "hooks")
     if not os.path.exists(hookdir):
         os.mkdir(hookdir)
@@ -180,8 +181,8 @@ def copy_hook(git_path: str, hook: str, ) -> None:
         return
     else:
         with resource_stream('gitprivacy.resources.hooks', hook) as src, dst:
-            shutil.copyfileobj(src, dst)
-            os.chmod(dst.fileno(), stat.S_IRWXU | stat.S_IRGRP | stat.S_IXGRP |
+            shutil.copyfileobj(src, dst)  # type: ignore
+            os.chmod(hook_fn, stat.S_IRWXU | stat.S_IRGRP | stat.S_IXGRP |
                      stat.S_IROTH | stat.S_IXOTH)  # mode 755
             print("Installed {} hook".format(hook))
 
@@ -279,12 +280,16 @@ def do_redate(ctx: click.Context, startpoint: str,
     single_commit = next(repo.head.commit.iter_parents(), None) is None
     try:
         if startpoint and not single_commit:
+            if not repo.is_ancestor(startpoint, "HEAD"):
+                click.echo("Startpoint not reachable from HEAD", err=True)
+                ctx.exit(128)
             rev = f"{startpoint}..HEAD"
         else:
             rev = "HEAD"
-            # Enforce validity of user-defined startpoint
-            # to give proper feedback
-            repo.commit(startpoint)
+            if startpoint:
+                # Enforce validity of user-defined startpoint
+                # to give proper feedback
+                repo.commit(startpoint)
         commits = list(repo.iter_commits(rev))
     except (git.GitCommandError, git.BadName):
         click.echo(f"bad revision '{startpoint}'", err=True)
@@ -300,7 +305,8 @@ def do_redate(ctx: click.Context, startpoint: str,
             err=True
         )
         ctx.exit(3)
-    with click.progressbar(commits, label="Redating commits") as bar:
+    # add commits in reversed order to rewriter startpoint first, HEAD last
+    with click.progressbar(reversed(commits), label="Redating commits") as bar:
         for commit in bar:
             rewriter.update(commit)
     rewriter.finish()
@@ -323,9 +329,11 @@ def redate_rewrites(ctx: click.Context):
     # determine commits to redate
     with open(rewrites_log_path, "r") as rwlog_fp:
         rwlog = list(map(_parse_post_rewrite_format, rwlog_fp))
-    olds = set(e[0] for e in rwlog)
-    news = set(e[1] for e in rwlog)
-    pending = news.difference(olds)  # ignore already rewritten news
+    olds_set = set(e[0] for e in rwlog)
+    news = [e[1] for e in rwlog]
+    news_set = set(news)
+    pending_set = news_set.difference(olds_set)  # ignore already rewritten news
+    pending = [noid for noid in news if noid in pending_set]
 
     if len(pending) == 0:
         click.echo("No pending rewrites to redact")
@@ -366,6 +374,10 @@ def do_check(ctx: click.Context):
         ctx.exit(2)
 
 
+def _sanitize_config_email(email: str) -> str:
+    return email.strip("'\"")
+
+
 @cli.command('tzcheck')
 @click.pass_context
 def check_timezone_changes(ctx: click.Context) -> bool:
@@ -374,7 +386,9 @@ def check_timezone_changes(ctx: click.Context) -> bool:
     if not repo.head.is_valid():
         return False  # no previous commits
     with repo.config_reader() as cr:
-        user_email = cr.get_value("user", "email", "")
+        user_email = _sanitize_config_email(
+            cr.get_value("user", "email", "")
+        )
     if not user_email:
         click.echo("No user email set.", err=True)
         ctx.exit(128)
@@ -384,6 +398,7 @@ def check_timezone_changes(ctx: click.Context) -> bool:
     )
     last_commit = next(user_commits, None)
     if last_commit is None:
+        click.echo("info: Skipping tzcheck - no previous commits with this email", err=True)
         return False  # no previous commits by this user
     current_tz = datetime.now(timezone.utc).astimezone().tzinfo
     if last_commit.author.email == user_email:
@@ -445,7 +460,7 @@ def _parse_post_rewrite_format(line: str) -> Tuple[str, str, str]:
     # <old-sha1> SP <new-sha1> [ SP <extra-info> ] LF
     vals = line.split(" ", maxsplit=2)
     n_vals = len(vals)
-    assert n_vals == 2 or n_vals == 3, "Unexpected post-rewrite format"
+    assert n_vals in (2, 3), "Unexpected post-rewrite format"
     if n_vals == 2:
         # pad to length 3
         vals.append("")
@@ -474,3 +489,4 @@ def _has_dirtydate(repo: git.Repo, redacter: DateRedacter,
 
 cli.add_command(email.redact_email)
 cli.add_command(keys.manage_keys)
+cli.add_command(pushcheck.check_push)
